//****************************************************
//***** AITENDO 128x48 SPI COLOR LCD controlling  **********
//*************************** By Minarai Shokunin ****
//*************************** 2014/04/13          ****
//****************************************************

#include "p24FJ64GA002.h"

// ************ defines ***************************

#define reset			LATBbits.LATB15
#define rs				LATBbits.LATB14
#define sda				LATBbits.LATB13
#define scl				LATBbits.LATB12
#define cs				LATBbits.LATB11

#define uchar unsigned char
#define uint unsigned int


// ************ prototypes ***************************

void LCD_CtrlWrite_IC(uchar);
void LCD_DataWrite_IC(uchar);  
void LCD_DataWrite(uchar,uchar);
void write_command(uchar);
void write_data(uchar);
void Reset();
void lcd_initial();
void RamAdressSet();
void PutPixel(uint,uint,uint);
void RedLine();
void dsp_single_colour(uchar,uchar);
void Output_Pixel(uint,uint);
void delay(uint);
void out_8bit(uchar);

// ************ Codes ***************************
void out_8bit(uchar dat){
	int i;
	uchar dat0;

	dat0 = dat;

	for (i=0;i<8;i++) {
		if ((dat0 & 0x80)==0){
			sda = 0;
		}else{
			sda =1;
		}
		scl = 0;
		dat0 <<=1;
		scl=1;
	}

}

void LCD_CtrlWrite_IC(uchar c)
{
cs=0;
rs=0;
	out_8bit(c);
cs=1;

}
void  LCD_DataWrite_IC(uchar d)  
{

cs=0;
rs=1;
	out_8bit(d);
cs=1;
}

void LCD_DataWrite(uchar LCD_DataH,uchar LCD_DataL)
{
LCD_DataWrite_IC(LCD_DataH);
LCD_DataWrite_IC(LCD_DataL);
}


void  write_command(uchar c)
{
cs=0;
rs=0;
	out_8bit(c);
cs=1;      
}

void  write_data(uchar d)
{
cs=0;
rs=1;
	out_8bit(d);
cs=1;
}


void Reset()
{
    reset=0;
    delay(100);
    reset=1;
    delay(100);
}
//////////////////////////////////////////////////////////////////////////////////////////////

void lcd_initial()
{

//------------------------------------------------------------------//  
//-------------------Software Reset-------------------------------//
write_command(0x11);//Sleep out
	delay(120);
	//ST7735R Frame Rate
	write_command(0xB1);
	write_data(0x01);
	write_data(0x2C);
	write_data(0x2D);
	write_command(0xB2);
	write_data(0x01);
	write_data(0x2C);
	write_data(0x2D);
	write_command(0xB3);
	write_data(0x01);
	write_data(0x2C);
	write_data(0x2D);
	write_data(0x01);
	write_data(0x2C);
	write_data(0x2D);
	//------------------------------------End ST7735R Frame Rate-----------------------------------------//
	write_command(0xB4);//Column inversion
	write_data(0x07);
	//------------------------------------ST7735R Power Sequence-----------------------------------------//
	write_command(0xC0);
	write_data(0xA2);
	write_data(0x02);
	write_data(0x84);
	write_command(0xC1);
	write_data(0xC5);
	write_command(0xC2);
	write_data(0x0A);
	write_data(0x00);
	write_command(0xC3);
	write_data(0x8A);
	write_data(0x2A);
	write_command(0xC4);
	write_data(0x8A);
	write_data(0xEE);
	//---------------------------------End ST7735R Power Sequence-------------------------------------//
	write_command(0xC5);//VCOM
	write_data(0x0E);
	write_command(0x36);//MX, MY, RGB mode
	write_data(0xC8);
	//------------------------------------ST7735R Gamma Sequence-----------------------------------------//
	write_command(0xe0);
	write_data(0x02);
	write_data(0x1c);
	write_data(0x07);
	write_data(0x12);
	write_data(0x37);
	write_data(0x32);
	write_data(0x29);
	write_data(0x2d);
	write_data(0x29);
	write_data(0x25);
	write_data(0x2b);
	write_data(0x39);
	write_data(0x00);
	write_data(0x01);
	write_data(0x03);
	write_data(0x10);
	write_command(0xe1);
	write_data(0x03);
	write_data(0x1d);
	write_data(0x07);
	write_data(0x06);
	write_data(0x2e);
	write_data(0x2c);
	write_data(0x29);
	write_data(0x2d);
	write_data(0x2e);
	write_data(0x2e);
	write_data(0x37);
	write_data(0x3f);
	write_data(0x00);
	write_data(0x00);
	write_data(0x02);
	write_data(0x10);
	 write_command(0x2A);
   write_data(0x00);
   write_data(0x02);
   write_data(0x00);
   write_data(0x81);
  
   write_command(0x2B);
   write_data(0x00);
   write_data(0x01);
   write_data(0x00);
   write_data(0xA0);
	//------------------------------------End ST7735R Gamma Sequence-----------------------------------------//

    write_command(0x3A); 
    write_data(0x05);  
	//write_command(0x3A);//65k mode
	//write_data(0x05);
	write_command(0x29);//Display on
}



void  RamAdressSet()
{
   write_command(0x2A);
   write_data(0x00);
   write_data(0x02);
   write_data(0x00);
   write_data(0x81);
  
   write_command(0x2B);
   write_data(0x00);
   write_data(0x01);
   write_data(0x00);
   write_data(0xA0);		  
}



void PutPixel(uint x,uint y,uint color)
{
    LCD_CtrlWrite_IC(0x2A);  //Column Address Set        ****
	LCD_DataWrite_IC(x);//xsh 
	LCD_DataWrite_IC(x);//xsl 	
	LCD_CtrlWrite_IC(0x2B);//Row Address Set             ****
	LCD_DataWrite_IC(y);//ysl 
	LCD_DataWrite_IC(y);//ysl 
	LCD_CtrlWrite_IC(0x2c);
	LCD_DataWrite(color>>8,color & 0xff);
    RamAdressSet();
}

void RedLine()
{
	uchar i=0,j=0;
	i=66;   // 11
	for(j=0;j<16;j++)
	{
		PutPixel(i,j,0xf800);
	}

}


void dsp_single_colour(uchar DH,uchar DL)
{
 uchar i,j;
 //RamAdressSet();
 for (i=0;i<160;i++)
    for (j=0;j<128;j++)
        LCD_DataWrite(DH,DL);
}


void Output_Pixel(uint x,uint y)
{
    LCD_CtrlWrite_IC(0x2A);  //Column Address Set        ****
	LCD_DataWrite_IC(x);//xsh 
	LCD_DataWrite_IC(x);//xsl 	
	LCD_CtrlWrite_IC(0x2B);//Row Address Set             ****
	LCD_DataWrite_IC(y);//ysl 
	LCD_DataWrite_IC(y);//ysl 
	LCD_CtrlWrite_IC(0x2c);
	LCD_DataWrite(0x00,0x00);
    RamAdressSet();

}


void delay(uint time)
{
 uint i,j;
  for(i=0;i<time;i++)
   for(j=0;j<250;j++);
}

